//Using this array you can set your ratios for outputs A to H
unsigned long ratios[8] = {
  1,  //A
  2,  //B
  4,  //C
  8,  //D
  16, //E
  3,  //F
  5,  //G
  7  //H
};

#include <Adafruit_NeoPixel.h>
#include <elapsedMillis.h>
#include <PCF8574.h>
#include <Wire.h>
#include "variables.h"

PCF8574 expander;
Adafruit_NeoPixel LEDs = Adafruit_NeoPixel(10, A3, NEO_RGB + NEO_KHZ800);
elapsedMillis time_to_switch;
elapsedMicros outs_micros[8];

uint32_t blue = LEDs.Color(0, 25, 255);
uint32_t green = LEDs.Color(0, 255, 25);
uint32_t off = LEDs.Color(0, 0, 0);


void setup() {
  Serial.begin(9600);

  delay(500);

  LEDs.begin();
  LEDs.setBrightness(35);
  LEDs.show();

  //Setup GPIO & interrupts
  pinMode(TRIG, INPUT);
  pinMode(RESET, INPUT);

  pinMode(2, INPUT);

  for (int i = 0; i < 8; i++) {
    pinMode(outs[i], OUTPUT);
  }

  //I2C expander setup
  expander.begin(0x20);
  expander.enableInterrupt(2, read_expander);

  for (int i = 0; i < 8; i++) {
    expander.pinMode(i, INPUT_PULLUP);
  }

  //setting the I2C clock to 400k, the maximum for the PCF8574
  Wire.setClock(400000L);

  attachInterrupt(INT1, trigger_int, CHANGE);

  for (int i = 0; i < 8; i++) {
    toggle_divide[toggle_index[i]] = expander.digitalRead(i);
    //Serial.print(toggle_divide[toggle_index[i]]);
    //Serial.print(" - ");
  }
}


void loop() {

  while (waiting_for_clock) {
    uint16_t i, j;
    for (j = 0; j < 256 * 5; j++) { // 5 cycles of all colors on wheel
      for (i = 0; i < LEDs.numPixels(); i++) {
        if (millis() - rainbow_wait > 5) {
          rainbow_wait = millis();
          LEDs.setPixelColor(i, Wheel(((i * 256 / LEDs.numPixels()) + j) & 255));
        }
        LEDs.show();
        //delay(wait);
        //Serial.println("still");
      }
      if (!waiting_for_clock) {
        break;
      }
    }

  }

  reset_actual = digitalRead(RESET);

  if (reset_actual && !reset_previous) {
    LEDs.setPixelColor(1, 0, 0, 0);
    show_leds = 1;
  }

  if (!reset_actual && reset_previous) {
    if (!do_we_know_the_period_yet) {
      if (reset_counter == 0) {
        reset_on_time = micros();
      }
      if (reset_counter == 1) {
        period = micros() - reset_on_time;
        do_we_know_the_period_yet = 1;
        //Serial.print("the period is ");
        //Serial.println(period);
      }
    }
    clock_counter = 0;
    reset_counter++;
    LEDs.setPixelColor(1, 0, 255, 0);
    show_leds = 1;
  }

  reset_previous = reset_actual;

  for (int i = 0; i < 8; i++) {

    //------TURNS OFF OUTPUT THAT ONLY DIVIDES------////
    //rountine to turn off the necessary outputs
    //when the ouputs were on for the clock_length
    //we turn them off
    if (toggle_divide[i]) {
      if (out_state[i]) {
        if (micros() - time_turned_on[i] > period / (2 * ratios[i])) {
          FastWrite(i, LOW);
          out_state[i] = 0;
          LEDs.setPixelColor(i + 2, off);
          show_leds = 1;
        }
      }
    }

    //------TURNS on and off Multiplied outputs------////
    //this takes care of all the mutliple clocks between clocks pulses
    //turn them on and off
    if (!toggle_divide[i]) {
      //turns ons on the output when we reach a new division of the input clock
      if (!out_state[i]) {
        if (outs_micros[i] > period / ratios[i]) {
          outs_micros[i] = 0;
          FastWrite(i, HIGH);
          out_state[i] = 1;
          LEDs.setPixelColor(i + 2, green);
          show_leds = 1;
        }
      }

      //with this next line we reset the clock when we reach 50% cycle
      if (out_state[i]) {
        if (outs_micros[i] > period / (2 * ratios[i])) {
          FastWrite(i, LOW);
          out_state[i] = 0;
          LEDs.setPixelColor(i + 2, off);
          show_leds = 1;
        }
      }
    }
  }

  if (show_leds) {
    show_leds = 0;
    LEDs.show();
  }

  if (read_toggles) {
    read_toggles = 0;
    for (int i = 0; i < 8; i++) {
      toggle_divide[toggle_index[i]] = expander.digitalRead(i);
      //Serial.print(toggle_divide[toggle_index[i]]);
      //Serial.print(" - ");
    }
    //Serial.println(" - ");
  }

}

void read_expander() {
  read_toggles = 1;
}

